	SUBROUTINE MAKEENV
******************************************************************* 
*  Finds cluster sites for every atom                             *
*  Essential input:                                               *
*    Rloc(isort) - cluster sphere radius for every SORT.          *
*  Outputs:                                                       *
*    Nenv(iatom) - # of nearest neighbors                         *
*    Tenv(1:3)   - vectors pointing to neighbors                  *
*    ISenv       - shows particular neighbor atoms                *
*    IPenv       - shows to which position atom will move         *
*                  after applying a group operation               *
*    IRenv       - shows irreducible connections                  *
*******************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL, ONLY : IASA,LMTO                       !!! from readini
      USE COM_PARAM,   ONLY : NATOM,NSORT,IS                  !!! from readini
      USE COM_LATPAR,  ONLY : PAR,ORTH                        !!! from readini
	USE COM_SPHERE,  ONLY : CLSMAX                          !!! from readini
      USE COM_LATTICE, ONLY : RBAS,GBAS,TAU                   !!! from readstr
      USE COM_SCRINP,  ONLY : RLOC                            !!! from alpcon
      USE COM_ENVSITE, ONLY : NENV,TENV,ISENV,IPENV,IRENV,ICENV!! from envmesh
     &                       ,NSHL,NATS,ISSHL          
      USE COM_SITEGEN, ONLY : NSITES,SITES,IREFSITES          !!! from sitegen
	USE COM_GROUPS,  ONLY : NGROUP,IKOV,IP,U,SHIFT          !!! from groups
      USE COM_MASHDEP, ONLY : LENINT,LENREL,LENCMP            !!! from setup
      USE COM_LABELS,  ONLY : TXTEL,TXTAT                     !!! from readini
      USE COM_UNIT,    ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER SIZE
	REAL*8, ALLOCATABLE :: TAUT(:,:)
	REAL*8, ALLOCATABLE :: RSHL(:,:)
	DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(NENV(NATOMMEM))
	ALLOCATE(NSHL(NATOMMEM))
	ALLOCATE(TAUT(3,NATOMMEM))
      WRITE(IUN,1001)
1001  FORMAT(1X,'Start generating cluster sites:'/)
C DEFINE CLUSTER SPHERE
      SMAX=0.D0
      DO IATOM=1,NATOM
      CALL SITEGEN_REDC(RBAS(1,1),RBAS(1,2),RBAS(1,3),
     &TAU(1,IATOM),TAUT(1,IATOM),0)
      S=SQRT(TAUT(1,IATOM)**2+TAUT(2,IATOM)**2+TAUT(3,IATOM)**2)
      SMAX=MAX(SMAX,S)
      ENDDO
	IF(ABS(SMAX).LE.1.D-10)SMAX=1.D0
	IF(ABS(CLSMAX).LE.1.D-10)CLSMAX=SMAX*1.5D0
	DO ISORT=1,NSORT
	IF(ABS(RLOC(ISORT)).LE.1.D-10)RLOC(ISORT)=CLSMAX
	RLOC(ISORT)=RLOC(ISORT)+1.D-07
	ENDDO
	CLSMAX=CLSMAX+1.D-07 ! increase a bit
C *** FIND CLUSTER SITES FOR ATOMS IN CELL ***
	DO 2000 ILOOP=1,2                      !!! loop twice
	IF(ILOOP.EQ.2)THEN
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(TENV(3,0:NENVMEM,NATOMMEM))
	ALLOCATE(ISENV(0:NENVMEM,NATOMMEM))
	ALLOCATE(IRENV(0:NENVMEM,NATOMMEM))
	ALLOCATE(IPENV(0:NENVMEM,NATOMMEM,NOPRMEM))
	ALLOCATE(ICENV(0:NENVMEM,0:NENVMEM,NATOMMEM))
	ALLOCATE(NATS(0:NENVMEM,NATOMMEM))
	ALLOCATE(ISSHL(NENVMEM+1,0:NENVMEM,NATOMMEM))
	ENDIF
	NENVMEM=-1
	DO 1000 IATOM=1,NATOM                  !!! over atoms
	ISORT=IS(IATOM)
	DO JATOM=1,NATOM
	TAUT(1,JATOM)=TAU(1,JATOM)-TAU(1,IATOM)
	TAUT(2,JATOM)=TAU(2,JATOM)-TAU(2,IATOM)
	TAUT(3,JATOM)=TAU(3,JATOM)-TAU(3,IATOM)
	ENDDO 
C GENERATE LATTICE SITES SURROUNDING ATOM "IATOM"
	CALL SITEGEN(RLOC(ISORT),RBAS,TAUT,NATOM)
	TX=SITES(1,0)
	TY=SITES(2,0)
	TZ=SITES(3,0)
	IM=IREFSITES(0)
	DO ISITES=0,NSITES
	TT=SQRT(SITES(1,ISITES)**2+
     &        SITES(2,ISITES)**2+
     &        SITES(3,ISITES)**2)
	IF(ABS(TT).LE.1.D-10)THEN
	SITES(1,0)=0.D0
	SITES(2,0)=0.D0
	SITES(3,0)=0.D0
	IREFSITES(0)=IATOM
	SITES(1,ISITES)=TX
	SITES(2,ISITES)=TY
	SITES(3,ISITES)=TZ
	IREFSITES(ISITES)=IM
	ENDIF
	ENDDO
C FIND ENVIRONMENT VECTORS
	IENV=-1
	DO ISITES=0,NSITES
	ABST=SQRT(SITES(1,ISITES)**2+
     &          SITES(2,ISITES)**2+
     &          SITES(3,ISITES)**2)
	IF(ABST.LE.RLOC(ISORT))THEN
	IENV=IENV+1
	IF(ILOOP.EQ.2)THEN                           !!! for iloop=2
	TENV(1,IENV,IATOM)=SITES(1,ISITES)
	TENV(2,IENV,IATOM)=SITES(2,ISITES)
	TENV(3,IENV,IATOM)=SITES(3,ISITES)
	ISENV(IENV,IATOM)=IREFSITES(ISITES)
	ENDIF                                        !!! for iloop=2
	ENDIF
	ENDDO
	NENV(IATOM)=IENV
	NENVMEM=MAX(NENV(IATOM),NENVMEM)
C MEMORY DEALLOCATIONS
	DEALLOCATE(SITES)
	DEALLOCATE(IREFSITES)
1000	CONTINUE                                     !!! over atoms
	IF(NENVMEM.EQ.-1)RETURN  ! no clusters generated
2000	CONTINUE                                     !!! over iloop
	DEALLOCATE(TAUT)
C ORDER VECTORS BY INCREASING LENGTH
	DO 3000 IATOM=1,NATOM                        !!! over atoms
      DO IENV=0,NENV(IATOM)
      ABSI=SQRT(TENV(1,IENV,IATOM)**2+TENV(2,IENV,IATOM)**2+
     &TENV(3,IENV,IATOM)**2)
      DO JENV=IENV+1,NENV(IATOM)
      ABSJ=SQRT(TENV(1,JENV,IATOM)**2+TENV(2,JENV,IATOM)**2+
     &TENV(3,JENV,IATOM)**2)
      IF(ABSJ.LT.ABSI)THEN
      XJ=TENV(1,JENV,IATOM)
      YJ=TENV(2,JENV,IATOM)
      ZJ=TENV(3,JENV,IATOM)
	ISENVJ=ISENV(JENV,IATOM)
      TENV(1,JENV,IATOM)=TENV(1,IENV,IATOM)
      TENV(2,JENV,IATOM)=TENV(2,IENV,IATOM)
      TENV(3,JENV,IATOM)=TENV(3,IENV,IATOM)
	ISENV(JENV,IATOM)=ISENV(IENV,IATOM)
      TENV(1,IENV,IATOM)=XJ
      TENV(2,IENV,IATOM)=YJ
      TENV(3,IENV,IATOM)=ZJ
	ISENV(IENV,IATOM)=ISENVJ
      ABSI=ABSJ
      ENDIF
	ENDDO
	ENDDO
3000	CONTINUE
C FIND SHELLS
	ALLOCATE(RSHL(0:NENVMEM,NATOMMEM))
	NSHLMEM=0
	DO 4000 IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	NSHL(IATOM)=0
	RSHL(0,IATOM)=0.D0
	DO IENV=1,NENV(IATOM)
      ABSI=SQRT(TENV(1,IENV,IATOM)**2+TENV(2,IENV,IATOM)**2+
     &TENV(3,IENV,IATOM)**2)
	ISHL=NSHL(IATOM)
	IF(ABSI.GT.RSHL(ISHL,IATOM)+1.D-10)THEN
     	NSHL(IATOM)=NSHL(IATOM)+1
	RSHL(NSHL(IATOM),IATOM)=ABSI
	ENDIF
	ENDDO
	NSHLMEM=MAX(NSHLMEM,NSHL(IATOM))
	DO ISHL=0,NSHL(IATOM)
	NATS(ISHL,IATOM)=0
	DO IENV=0,NENV(IATOM)
      ABSI=SQRT(TENV(1,IENV,IATOM)**2+TENV(2,IENV,IATOM)**2+
     &TENV(3,IENV,IATOM)**2)
	IF(ABS(RSHL(ISHL,IATOM)-ABSI).LE.1.D-10)THEN
      NATS(ISHL,IATOM)=NATS(ISHL,IATOM)+1
	ISSHL(NATS(ISHL,IATOM),ISHL,IATOM)=IENV
	ENDIF
	ENDDO
	ENDDO
4000	CONTINUE
	DEALLOCATE(RSHL)
C  *** FIND SYMMETRY OF THE CLUSTER SITES ****
	DO 5000 IATOM=1,NATOM              !!! over atoms
	DO IENV=0,NENV(IATOM)              !!! over n.neigbors
C  APPLY INVERSIONAL GROUP OPERATION g^{-1}*T=r^{-1}*T-r^{-1}*shift
	DO IR=1,NGROUP
	IG=IKOV(IR)
C  Rotate shift vector if group is non-symmorfical
      SX=SHIFT(1,IR)
      SY=SHIFT(2,IR)
      SZ=SHIFT(3,IR)
      CALL ROTATE(SX,SY,SZ,XS,YS,ZS,U(2,IG),1)
C  Rotate basis vector
	TX=TAU(1,IATOM)
	TY=TAU(2,IATOM)
	TZ=TAU(3,IATOM)
      CALL ROTATE(TX,TY,TZ,XT,YT,ZT,U(2,IG),1)
      TXR=XT-XS
      TYR=YT-YS
      TZR=ZT-ZS
C  Rotate neighbor
	UX=TENV(1,IENV,IATOM)+TAU(1,IATOM)
	UY=TENV(2,IENV,IATOM)+TAU(2,IATOM)
	UZ=TENV(3,IENV,IATOM)+TAU(3,IATOM)
      CALL ROTATE(UX,UY,UZ,XU,YU,ZU,U(2,IG),1)
      UXR=XU-XS
      UYR=YU-YS
      UZR=ZU-ZS
C  NOW, ROTATED NEIGHBOR IS
	EXR=UXR-TXR
	EYR=UYR-TYR
	EZR=UZR-TZR
C  FIND IT IN THE ARRAY TENV:::
	IATOM1=IP(IATOM,IR)
	DO IENV1=0,NENV(IATOM1)
	DIFF=(EXR-TENV(1,IENV1,IATOM1))**2+
     &     (EYR-TENV(2,IENV1,IATOM1))**2+
     &     (EZR-TENV(3,IENV1,IATOM1))**2
	IF(DIFF.LE.1.D-10)EXIT
	ENDDO
	IF(IENV1.EQ.NENV(IATOM1)+1)CALL MAKEENV_MESG01
	IPENV(IENV,IATOM,IR)=IENV1
C  CHECK FINDING
	JATOM=ISENV(IENV,IATOM)
	JATOMG=IP(JATOM,IR)
	JATOM1=ISENV(IENV1,IATOM1)
	IF(JATOM1.NE.JATOMG)CALL MAKEENV_MESG02
	ENDDO                              !!! over group
	ENDDO                              !!! over n.neighbors
5000	CONTINUE                           !!! over atoms
C  SET IRREDUCIBLE VECTORS
	DO IATOM=1,NATOM
	DO IENV=0,NENV(IATOM)
	IRENV(IENV,IATOM)=0
	ENDDO
	ENDDO
	DO IATOM=1,NATOM                   !!! over atoms
	DO IENV=0,NENV(IATOM)              !!! over n.neigbors
	IF(IRENV(IENV,IATOM).EQ.0)THEN
	IRENV(IENV,IATOM)=1
	DO IR=1,NGROUP
	IENV1=IPENV(IENV,IATOM,IR)
	IATOM1=IP(IATOM,IR)
	IF(.NOT.(IENV1.EQ.IENV.AND.IATOM1.EQ.IATOM))
     &  IRENV(IENV1,IATOM1)=-1
	ENDDO
	ENDIF
	ENDDO
	ENDDO
C  CHECK OUT GENERATION AND PRINT DATA
 	WRITE(IUN,1010)
 	DO IATOM=1,NATOM
 	ISORT=IS(IATOM)
      WRITE(IUN,1020)(TAU(I,IATOM),I=1,3),TXTEL(ISORT)
 	WRITE(IUN,1030)NSHL(IATOM),RLOC(ISORT)
 	NENV0=0
 	DO ISHL=1,NSHL(IATOM)
 	WRITE(IUN,1040)ISHL,NATS(ISHL,IATOM)
 	DO IATS=1,NATS(ISHL,IATOM)
 	IENV=ISSHL(IATS,ISHL,IATOM)
 	IATOMI=ISENV(IENV,IATOM)
 	ISORTI=IS(IATOMI)
 	DIST=SQRT(TENV(1,IENV,IATOM)**2+TENV(2,IENV,IATOM)**2+
     &TENV(3,IENV,IATOM)**2)
 	WRITE(IUN,1050)(TENV(I,IENV,IATOM),I=1,3),DIST,
     &TXTEL(ISORTI),IATOMI
 	NENV0=NENV0+1
 	ENDDO
 	ENDDO
 	IF(NENV0.NE.NENV(IATOM))CALL MAKEENV_MESG03
 	ENDDO
1010  FORMAT(1X,'Result from MAKEENV for the nearest neighbors   --->')
1020  FORMAT(1X,'Position : (',G12.5,',',G12.5,',',G12.5,') ',A10)
1030  FORMAT(1X,'# of shells found ',I3,' for loc. radius ',G12.5)
1040  FORMAT(1X,'# of atoms in shell ',I3,' is ',I3,'. Neighbors::')
1050  FORMAT(1X,'(',G12.5,',',G12.5,',',G12.5,') ',
     &'d=',G12.5,1X,A10,1X,I3)
C  PRINT IRREDUCIBLE CONNECTIONS ONSITE
	NB=1
	DO ISORT=1,NSORT
      DO I=10,5,-1
      IF(TXTEL(ISORT)(I:I).NE.' ')EXIT
      ENDDO
      NB=MAX(NB,I)
	ENDDO
 	WRITE(IUN,1100)
	ISITE=0
	DO IATOM=1,NATOM
	DO IENV=0,0          
	IF(IRENV(IENV,IATOM).EQ.1)ISITE=ISITE+1
	ENDDO
	ENDDO
	WRITE(IUN,1110)ISITE
	WRITE(IUN,1130)
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO IENV=0,0          
	IATOM1=ISENV(IENV,IATOM)
	ISORT1=IS(IATOM1)
	IF(IRENV(IENV,IATOM).EQ.1)WRITE(IUN,1140)
     &TXTEL(ISORT)(5:NB),IATOM,TXTEL(ISORT1)(5:NB),IATOM1,
     &TENV(1,IENV,IATOM),TENV(2,IENV,IATOM)/ORTH(2),
     &TENV(3,IENV,IATOM)/ORTH(3)
	ENDDO
	ENDDO
C  PRINT IRREDUCIBLE CONNECTIONS BETWEEN SITES
	IHOPP=0
	DO IATOM=1,NATOM
	DO IENV=1,NENV(IATOM)
	IF(IRENV(IENV,IATOM).EQ.1)IHOPP=IHOPP+1
	ENDDO
	ENDDO
	WRITE(IUN,1120)IHOPP
	WRITE(IUN,1130)
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO IENV=1,NENV(IATOM)
	IATOM1=ISENV(IENV,IATOM)
	ISORT1=IS(IATOM1)
	IF(IRENV(IENV,IATOM).EQ.1)WRITE(IUN,1140)
     &TXTEL(ISORT)(5:NB),IATOM,TXTEL(ISORT1)(5:NB),IATOM1,
     &TENV(1,IENV,IATOM),TENV(2,IENV,IATOM)/ORTH(2),
     &TENV(3,IENV,IATOM)/ORTH(3)
	ENDDO
	ENDDO
1100  FORMAT(1X,'Result from MAKEENV for irreducible connections --->')
1110  FORMAT(1X,'Intrasite irreducible connections found:',I4)
1120  FORMAT(1X,'Intersite irreducible connections found:',I4)
1130  FORMAT(1X,'From:',10X,'to:',12X,'via vector:')
1140  FORMAT(1X,A,'#',I2,10X,A,'#',I2,10X,'(',G12.5,',',G12.5,
     &',',G12.5,')')
C FIND POINT-INDICATOR ARRAY FOR THREE-CENTER TERMS
      DO 6000 IATOM1=1,NATOM
      DO IENV=0,NENV(IATOM1)
      IATOM=ISENV(IENV,IATOM1)
      TXI=TAU(1,IATOM1)+TENV(1,IENV,IATOM1)
      TYI=TAU(2,IATOM1)+TENV(2,IENV,IATOM1)
      TZI=TAU(3,IATOM1)+TENV(3,IENV,IATOM1)
      DO KENV=0,NENV(IATOM1)
      KATOM=ISENV(KENV,IATOM1)
      TXK=TAU(1,IATOM1)+TENV(1,KENV,IATOM1)
      TYK=TAU(2,IATOM1)+TENV(2,KENV,IATOM1)
      TZK=TAU(3,IATOM1)+TENV(3,KENV,IATOM1)
      ICENV(KENV,IENV,IATOM1)=-1
      DO LENV=0,NENV(IATOM)
      TXL=TXI+TENV(1,LENV,IATOM)
      TYL=TYI+TENV(2,LENV,IATOM)
      TZL=TZI+TENV(3,LENV,IATOM)
      ABSR=SQRT((TXL-TXK)**2+(TYL-TYK)**2+(TZL-TZK)**2)
      IF(ABSR.LE.1.D-08)ICENV(KENV,IENV,IATOM1)=LENV
	ENDDO
	ENDDO
	ENDDO
C CHECK OUT FOR SYMMETRICITY
      DO IENV=0,NENV(IATOM1)
      IATOM=ISENV(IENV,IATOM1)
      DO KENV=0,NENV(IATOM1)
      KATOM=ISENV(KENV,IATOM1)
      LENV=ICENV(KENV,IENV,IATOM1)
      IF(LENV.GE.0)THEN
      LATOM=ISENV(LENV,IATOM)
      IF(LATOM.NE.KATOM)CALL MAKEENV_MESG04
      ENDIF
      IF((ICENV(KENV,IENV,IATOM1).EQ.-1.
     &AND.ICENV(IENV,KENV,IATOM1).NE.-1).OR.
     &   (ICENV(IENV,KENV,IATOM1).EQ.-1.
     &AND.ICENV(KENV,IENV,IATOM1).NE.-1.))CALL MAKEENV_MESG05
	ENDDO
	ENDDO
6000  CONTINUE
C ERROR HANDLINGS
      CONTAINS
      SUBROUTINE MAKEENV_MESG01
      WRITE(IUT,*)'Terminated in <MAKEENV>:'
      WRITE(IUT,*)'Internal error in generating cluster sites.'
      WRITE(IUT,*)'Cluster sites do not obey symmetry.'
	WRITE(IUT,*)'Check out algorithms in MAKEENV/SITEGEN.'
	CALL FULLSTOP(' ')
      END SUBROUTINE MAKEENV_MESG01
      SUBROUTINE MAKEENV_MESG02
      WRITE(IUT,*)'Terminated in <MAKEENV>:'
      WRITE(IUT,*)'Internal error in generating cluster sites.'
      WRITE(IUT,*)'Atoms in the culster sites mismatch.'
	WRITE(IUT,*)'Check out algorithms.'
	CALL FULLSTOP(' ')
      END SUBROUTINE MAKEENV_MESG02
      SUBROUTINE MAKEENV_MESG03
      WRITE(IUT,*)'Terminated in <MAKEENV>:'
      WRITE(IUT,*)
     &'Internal error. Wrong generation of shells detected.'
	WRITE(IUT,*)'Check out algorithms in MAKEENV/SITEGEN.'
	CALL FULLSTOP(' ')
      END SUBROUTINE MAKEENV_MESG03
      SUBROUTINE MAKEENV_MESG04
      WRITE(IUT,*)'Terminated in <MAKEENV>:'
      WRITE(IUT,*)'Internal error. Wrong ICENV array.'
	WRITE(IUT,*)'Check out algorithm in MAKEENV/SITEGEN.'
	CALL FULLSTOP(' ')
      END SUBROUTINE MAKEENV_MESG04
      SUBROUTINE MAKEENV_MESG05
      WRITE(IUT,*)'Warning from  <MAKEENV>:'
      WRITE(IUT,*)'Symmetry is broken in the cluster:'
	WRITE(IUT,*)'Kenv,Ienv,Iatom ==',KENV,IENV,IATOM1
	CALL FULLSTOP(' ')
      END SUBROUTINE MAKEENV_MESG05
      END SUBROUTINE MAKEENV
