	SUBROUTINE SERFLL(ERFL,LMAX,ETA,R,E)
***********************************************************
*  Calculates integral over semiinfinite region needed for*
*  evaluating the Evald lattice sum                       *
*                                                         *
*           infinity                                      *
*              /                                          *
*     ERFL(L)= \ x**(2*l)*exp(-(R*x)**2+E/4x**2)*dx/f     *
*              /                                          *
*             eta                                         *
*                                                         *
*  where f=eta**2l*exp(-(R*eta)**2+E/4eta**2)             *
*  Method: Tailor series over E/4x**2 and recursion       *
*  relations for positive (possibly complex) energies.    *
***********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 ERFL(-1:LMAX),E,BETTA,ENERG,SUM0,SUM1
     &    ,FACE,FACX,TER0,TER1
	DIMENSION FACN(5000),FAC1N(5000)
	LOGICAL FIRST
	SAVE FACN,FAC1N
	DATA SQPI14/0.88622692545275D0/,EPS/1.D-10/,FIRST/.TRUE./
	IF(FIRST)THEN
	DO 1 N=1,5000
	FACN(N)=1.D0/DBLE(N)
	FAC1N(N)=1.D0/DBLE(2*N+1)
   1  CONTINUE
	FIRST=.FALSE.
	ENDIF
	DELTA=R*ETA
	GAMMA=EXP(DELTA*DELTA)
	ENERG=0.25D0*E/(ETA*ETA)
	BETTA=EXP(-ENERG)
C  DO FOR POSITIVE ENERGIES OR FOR NEGATIVE COMPLEX ENERGIES
	IF(DREAL(E).GE.0.D0.OR.(DREAL(E).LT.0.D0.AND.
     &   DIMAG(E).NE.0.D0))THEN
C  CALCULATE STANDARD ERROR FUNCTION
	DERFF=ERFC(DELTA)
	IF(DERFF.EQ.0.D0)GOTO40
	R2FAC=2.D0*R*R*ETA
	FNP0=SQPI14*DERFF*GAMMA/R
	FNP1=ETA*(1.D0-R2FAC*FNP0)
C  CALCULATE ERF(L) FOR L=0 AND -1 USING RECURSION RELATIONS
	SUM0=FNP0*BETTA
	SUM1=FNP1*BETTA
	FNP0=FNP1
	FACE=(1.D0,0.D0)
	DO 10 N=1,5000
	FACE=FACE*ENERG*FACN(N)
	FNP1=ETA*FAC1N(N)*(1.D0-R2FAC*FNP0)
	FACX=FACE*BETTA
	TER0=FACX*FNP0
	TER1=FACX*FNP1
	SUM0=SUM0+TER0
	SUM1=SUM1+TER1
	IF(ABS(TER0/SUM0).LE.EPS)GOTO20
	FNP0=FNP1
  10  CONTINUE
	PRINT*,'<ERFLL>: NO CONVERGENCY AFTER 5000 ITERATIONS, ',TER0
  20  ERFL( 0)=SUM0
	ERFL(-1)=SUM1
C  DO FOR NEGATIVE REAL ENERGIES
	ELSE
	AKAPA=SQRT(-DREAL(E))
	ALPHA=AKAPA/2.D0/ETA
	THETA=EXP(-AKAPA*R)
	IF(THETA.EQ.0.D0)GOTO40
	UPLUS=-ERFC(DELTA-ALPHA)*THETA
	UMINS=+ERFC(ALPHA+DELTA)/THETA
	ERFL( 0)=-SQPI14*(UPLUS-UMINS)*GAMMA*BETTA/R/2.D0
	ERFL(-1)=-SQPI14*(UPLUS+UMINS)*GAMMA*BETTA/AKAPA*ETA*ETA
	ENDIF
C  CALCULATE ERF(L) FOR ALL L USING RECURSION RELATIONS
	ETAN=1.D0/ETA
	ETA3N=ETAN**3
	R2ETA=1.D0/(2.D0*R*R*ETA)
	DO 30 L=0,LMAX-1
	L2=2*L+1
	ERFL(L+1)=R2ETA*(1.D0+L2*ERFL(L)*ETAN-0.5D0*E*ETA3N*ERFL(L-1))
  30  CONTINUE
	RETURN
C  NO ACCURACY IN THE ERFC, SET VALUES TO ZERO
  40  DO 50 L=-1,LMAX
	ERFL(L)=(0.D0,0.D0)
  50  CONTINUE
	END




	SUBROUTINE SERFL1(ERFL,LMAX,ETA,R,E)
***********************************************************
*  Calculates integral over semiinfinite region needed for*
*  evaluating the Evald lattice sum                       *
*                                                         *
*           infinity                                      *
*              /                                          *
*     ERFL(L)= \ x**(2*l)*exp(-(R*x)**2+E/4x**2)*dx/f     *
*              /                                          *
*             eta                                         *
*                                                         *
*  where f=eta**2l*exp(-(R*eta)**2+E/4eta**2)             *
*  Method: Expressions via complex error function         *
*  for all (possibly complex) energies.                   *
***********************************************************
	IMPLICIT REAL*8 (A,B,D-H,O-Z)
	IMPLICIT COMPLEX*16 (C)
	COMPLEX*16 ERFL(-1:LMAX),E,BETTA,ENERG
	DATA SQPI14/0.88622692545275D0/
	DELTA=R*ETA
	GAMMA=EXP(DELTA*DELTA)
	ENERG=0.25D0*E/(ETA*ETA)
	BETTA=EXP(-ENERG)
	IF(DREAL(E).GE.0.D0)CKAPA=+SQRT(E)
	IF(DREAL(E).LT.0.D0)CKAPA=-SQRT(E)
	CLPHA=CKAPA/2.D0/ETA
	CZPLS=-CLPHA+(0.D0,1.D0)*DELTA
	CZMNS=+CLPHA+(0.D0,1.D0)*DELTA
	CPLUS=CWERF(CZPLS)
	CMINS=CWERF(CZMNS)
	ERFL( 0)=SQPI14*(CPLUS+CMINS)/2.D0/R
	IF(ABS(CKAPA).GE.1.D-10)
     &  ERFL(-1)=SQPI14*(CPLUS-CMINS)*(0.D0,1.D0)/CKAPA*ETA*ETA
	IF(ABS(CKAPA).LT.1.D-10)
     &  ERFL(-1)=ETA*(1.D0-2.D0*SQPI14*DELTA*CPLUS)
C  CALCULATE ERF(L) FOR ALL L USING RECURSION RELATIONS
	ETAN=1.D0/ETA
	ETA3N=ETAN**3
	R2ETA=1.D0/(2.D0*R*R*ETA)
	DO 30 L=0,LMAX-1
	L2=2*L+1
	ERFL(L+1)=R2ETA*(1.D0+L2*ERFL(L)*ETAN-0.5D0*E*ETA3N*ERFL(L-1))
  30  CONTINUE
	RETURN
C  NO ACCURACY IN THE ERFC, SET VALUES TO ZERO
  40  DO 50 L=-1,LMAX
	ERFL(L)=(0.D0,0.D0)
  50  CONTINUE
	END




	SUBROUTINE SERFL2(ERFL,LMAX,ETA,R,E)
***********************************************************
*  Calculates contour integral                            *
*                                                         *
*             eta                                         *
*              /                                          *
*     ERFL(L)= \ x**(2*l)*exp(-(R*x)**2+E/4x**2)*dx/f     *
*              /                                          *
*             0Co                                         *
*                                                         *
*  where f=eta**2l*exp(-(R*eta)**2+E/4eta**2)             *
*  Method: Expressions via complex error function         *
*  for all (possibly complex) energies and iterative      *
*  recursion relations.                                   *
***********************************************************
	IMPLICIT REAL*8 (A,B,D-H,O-Z)
	IMPLICIT COMPLEX*16 (C)
	COMPLEX*16 ERFL(-1:LMAX),E,BETTA,ENERG
     &   ,ERFT(-1:1000),ERFT1(-1:1000)
	DATA SQPI14/0.88622692545275D0/,ACCU/1.D-10/
	DELTA=R*ETA
	GAMMA=EXP(DELTA*DELTA)
	ENERG=0.25D0*E/(ETA*ETA)
	BETTA=EXP(-ENERG)
	IF(DREAL(E).GE.0.D0)CKAPA=+SQRT(E)
	IF(DREAL(E).LT.0.D0)CKAPA=-SQRT(E)
	CLPHA=CKAPA/2.D0/ETA
	CZPLS=-CLPHA-(0.D0,1.D0)*DELTA
	CZMNS=-CLPHA+(0.D0,1.D0)*DELTA
	CPLUS=CWERF(CZPLS)
	CMINS=CWERF(CZMNS)
C VALUE FOR L=0
	IF(ABS(R).GE.1.D-10)THEN
	ERFL(0)=SQPI14*(CPLUS-CMINS)/2.D0/R
	IF(DREAL(E).GE.0.D0)
     &ERFL(0)=ERFL(0)+GAMMA*BETTA*SQPI14*(0.D0,1.D0)*SIN(CKAPA*R)/R
	ELSEIF(ABS(R).LT.1.D-10)THEN
	ERFL(0)=ETA-(0.D0,1.D0)*SQPI14*CKAPA*CPLUS
	IF(DREAL(E).GE.0.D0)
     &ERFL(0)=ERFL(0)+BETTA*SQPI14*(0.D0,1.D0)*CKAPA
	ENDIF
C VALUE FOR L=-1
	IF(ABS(E).GE.1.D-10)THEN
	ERFL(-1)=-SQPI14*(CPLUS+CMINS)*(0.D0,1.D0)/CKAPA*ETA*ETA
	IF(DREAL(E).GE.0.D0)
     &ERFL(-1)=ERFL(-1)+ETA*ETA*GAMMA*BETTA*2.D0*SQPI14*(0.D0,1.D0)*
     &COS(CKAPA*R)/CKAPA
	ELSEIF(ABS(E).LT.1.D-10)THEN
	ERFL(-1)=-ETA*(1.D0+SQPI14*2.D0*DELTA*GAMMA*ERF(DELTA))
	ENDIF
C  CALCULATE ERF(L) FOR ALL L USING RECURSION RELATIONS
	ETAN=1.D0/ETA
	ETA3N=ETAN**3
	R2ETA=2.D0*R*R*ETA
C  ESTIMATE CUTOFF L FOR REVERSE RECURSION
	ALIM=0.05D0
	RLIM=0.5D0*(2.D0*R*R*ETA*ETA/ALIM-1.D0)
	ELIM=0.5D0*(0.5D0*ABS(E)/ETA/ETA/ALIM-1.D0)
	LCUT1=ANINT(RLIM)
	LCUT2=ANINT(ELIM)
	LCUT=MAX(LCUT1,LCUT2)
	IF(LCUT.LT.LMAX)LCUT=LMAX
	IF(LCUT.LT.50)LCUT=50
	IF(LCUT.GT.1000)LCUT=1000
C  ESTIMATE KAPPA*R FOR DIRECT RECURSION
	EREL=DREAL(E)
	IF(EREL.LE.0.D0)XMAX=SQRT(-EREL)*R
	IF(EREL.GT.0.D0)XMAX=SQRT(+EREL)*R
	XCUT=1.D0
C  IF ENERGY IS EQUAL TO ZERO, R IS ARBITRARY
	IF(ABS(E).LT.1.D-10)THEN
C  SET UP ERROR FUNCTION "ERFL" FOR E=0 BY REVERSE RECURSION
	ERFT(LCUT)=ETA/DBLE(2*LCUT+1)
	DO 10 L=LCUT-1,0,-1
	L2=2*L+1
	ERFT(L)=ETA/DBLE(L2)*(R2ETA*ERFT(L+1)+1.D0)
	IF(L.LE.LMAX)ERFL(L)=ERFT(L)
  10  CONTINUE
C  IF R IS EQUAL TO ZERO, E IS ARBITRARY
	ELSEIF(ABS(R).LE.1.D-10)THEN
C  SET UP ERROR FUNCTION "ERFL" FOR R=0 BY DIRECT RECURSION
	DO 20 L=1,LMAX
	L2=2*L+1
	ERFL(L)=(1.D0+0.5D0*E*ETA3N*ERFL(L-1))/DBLE(L2)*ETA
 20   CONTINUE
C IF R IS NOT ZERO AND E IS NOT ZERO
	ELSEIF(XMAX.LE.XCUT)THEN
C  SET UP ERROR FUNCTION "ERFT1" FOR E=0 BY REVERSE RECURSION
	ERFT1(LCUT)=ETA/DBLE(2*LCUT+1)
	ERFT1(-1)=ERFL(-1)
	DO 30 L=LCUT-1,0,-1
	L2=2*L+1
	ERFT1(L)=ETA/DBLE(L2)*(R2ETA*ERFT1(L+1)+1.D0)
C     ERFT1(L)=ETA/DBLE(2*L+1)
  30  CONTINUE
C  SET UP ERROR FUNCTION "ERFT1" FOR R=0 BY DIRECT RECURSION
C     ERFT1(-1)=ERFL(-1)
C     DO 30 L=0,LCUT
C     L2=2*L+1
C     ERFT1(L)=(1.D0+0.5D0*E*ETA3N*ERFT1(L-1))/DBLE(L2)*ETA
C 30  CONTINUE
C  SET UP ERROR FUNCTION "ERFT" BY ITERATIVE REVERSE RECURSION
	DO 70 IT=1,1000
	ERFT(LCUT)=ETA/DBLE(2*LCUT+1)
	DO 40 L=LCUT-1,0,-1
	L2=2*L+1
	ERFT(L)=ETA/DBLE(L2)*(R2ETA*ERFT(L+1)+1.D0+
     &  0.5D0*E*ETA3N*ERFT1(L-1))
  40  CONTINUE
	IEXIT=0             
	DO 50 L=0,LMAX
	IF(ABS(ERFT(L)).EQ.0.D0)IEXIT=IEXIT+1
	IF(ABS(ERFT(L)).EQ.0.D0)GOTO50
	IF(ABS((ERFT1(L)-ERFT(L))/ERFT(L)).LE.ACCU)IEXIT=IEXIT+1
  50  CONTINUE
	IF(IEXIT.EQ.LMAX+1)GOTO80
	DO 60 L=0,LCUT
	ERFT1(L)=ERFT(L)
  60  CONTINUE
  70  CONTINUE
	CALL FULLSTOP('<SERFL2> ITERATIVE RECURSION DIVERGES')
C  CHECK OUT WITH THE CALCULATED ERFL(0)
  80  IF(ABS(ERFL(0)).EQ.0.D0)GOTO 90
	IF(ABS((ERFT(0)-ERFL(0))/ERFL(0)).GT.10.D0*ACCU.AND.R.GT.0.5D0)
     &PRINT*,'ACCURACY IS LOW IN <SERFL2>'
  90  DO 100 L=0,LMAX
	ERFL(L)=ERFT(L)
 100  CONTINUE
	ELSEIF(XMAX.GT.XCUT)THEN
	DO 110 L=0,LMAX-1
	L2=2*L+1
	ERFL(L+1)=(-1.D0+DBLE(L2)*ERFL(L)*ETAN-
     &  0.5D0*E*ETA3N*ERFL(L-1))/R2ETA
 110  CONTINUE
	ENDIF
	END




	SUBROUTINE SERFL3(ERFL,LMAX,ETA,R,E,CUTK)
***********************************************************
*  Calculates contour integral                            *
*                                                         *
*             eta                                         *
*              /                                          *
*     ERFL(L)= \ x**(2*l)*exp(-(R*x)**2+E/4x**2)*dx/f     *
*              /                                          *
*             0Co                                         *
*                                                         *
*  where f=eta**2l*exp(-(R*eta)**2+E/4eta**2)             *
*  using reciprocal-space representation [up to "cutk"].  *
***********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 E,ERFL(-1:LMAX),FUNK,SUMF,OLDF
	DIMENSION BES(0:30),BESD(0:30)
	DATA PI/3.141592657589793D0/
C  FIRST CALCULATE IT DIRECTLY
	CALL SERFL2(ERFL,LMAX,ETA,R,E)
C  ESTIMATE ERROR AS INT{CUTK,INF} BY SIMPLE INTEGRATION
	H=0.01D0
	RINF=1000.D0
	DO 30 L=0,LMAX
	COFL=2.D0/PI/ETA**(2*L)*EXP(R**2*ETA**2)/2.D0**(L+1)
	SUMF=(0.D0,0.D0)
	OLDF=(0.D0,0.D0)
	DO 10 PK=CUTK,RINF,H
	IF(L.EQ.0)PKL=1.D0
	IF(L.NE.0)PKL=PK**L
	IF(R.GE.1.D-12)THEN
	CALL BESSR(PK*R,BES,BESD,L)
	BESRL=BES(L)/R**L
	ELSEIF(R.LT.1.D-12)THEN
	BESRL=PKL/FAC2(2*L+1)
	ENDIF
	IF(ABS(E).GE.1.D-10)THEN
	FUNK=BESRL*PK**(L+2)/(PK**2-E)*EXP((-PK**2)/4.0/ETA/ETA)
	ELSEIF(ABS(E).LT.1.D-10)THEN
	FUNK=BESRL*PKL*EXP((-PK**2)/4.0/ETA/ETA)
	ENDIF
	SUMF=SUMF-COFL*FUNK*H
	IF(ABS((OLDF-SUMF)/SUMF).LE.1.D-12)GOTO20
	OLDF=SUMF
  10  CONTINUE
  20  ERFL(L)=ERFL(L)-SUMF
  30  CONTINUE
	END





	FUNCTION CWERF(Z) 
***************************************************************
* Calculates complex error function w(z) as defined in        *
* M. Abramowitz and I.A. Stegun, (Russ. transl. 1979, p.120): *
*                 w(z)=exp(-z^2)*erfc(-iz)       [1]          *
* To calculate complex erf(z) just write:                     *
*                 erf(z)=1-exp(-z^2)*w(iz)       [2]          *
* where "i" is an imaginary unit.                             *
* Taken from CERN Library, adapted to double precision.       *
***************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 Z,CWERF
      REAL*8 LAMBDA   
      LOGICAL B 
      XX=DREAL(Z)    
      YY=DIMAG(Z)   
      X=ABS(XX) 
      Y=ABS(YY) 
      IF(Y .LT. 4.29D0 .AND. X .LT. 5.33D0) GO TO 1 
      H=0.D0
      NC=0  
      NU=8  
      LAMBDA=0.D0
      B=.TRUE.  
      GO TO 2   
    1 S=(1.D0-Y/4.29D0)*SQRT(1.D0-X**2/28.41D0)   
      H=1.6D0*S   
      H2=2.D0*H  
      NC=6+INT(23.D0*S)  
      NU=9+INT(21.D0*S)  
      LAMBDA=H2**NC 
      B= LAMBDA .EQ. 0.D0
    2 R1=0.D0
      R2=0.D0 
      S1=0.D0 
      S2=0.D0 
      N=NU+1    
    3 N=N-1 
      FN=N+1    
      T1=Y+H+FN*R1  
      T2=X-FN*R2    
      C=0.5D0/(T1**2+T2**2)   
      R1=C*T1   
      R2=C*T2   
      IF(H .LE. 0.D0 .OR. N .GT. NC) GO TO 4 
      T1=LAMBDA+S1  
      S1=R1*T1-R2*S2    
      S2=R2*T1+R1*S2    
      LAMBDA=LAMBDA/H2  
    4 IF(N .GT. 0) GO TO 3  
      IF(B) GO TO 6 
      RS1=S1    
      RS2=S2    
      GO TO 7   
    6 RS1=R1    
      RS2=R2    
    7 RS1=1.12837 91670 9551D0 *RS1   
      IF(Y .EQ. 0.D0) RS1=EXP(-X**2) 
      CWERF=DCMPLX(RS1,1.12837 91670 9551D0 *RS2)  
      IF(YY .LT. 0.D0) GO TO 8   
      IF(XX .LT. 0.D0) CWERF=CONJG(CWERF)    
      RETURN    
    8 CWERF=2.D0*EXP(-DCMPLX(X,Y)**2)-CWERF  
      IF(XX .GT. 0.D0) CWERF=CONJG(CWERF)    
      RETURN    
      END   



      FUNCTION ERFXX(X)
C
C     ******************************************************************
C
C     ENTRY POINTS ... ERF, ERFC, FREQ.
C
C     THESE THREE FUNCTIONS ARE COMPUTED FROM THE RATIONAL APPROXIMAT-
C     IONS OF W.J.CODY, MATHEMATICS OF COMPUTATION, VOLUME 22 (1969),
C     PAGES 631-637.
C
C     FOR ABS(X) LE 0.47 THE BASIC FUNCTION IS ERF.  FOR ABS(X) GT 0.47 
C     THE BASIC FUNCTION IS ERFC.  THE FINAL RESULT IS OBTAINED IN TERMS
C     OF THE BASIC FUNCTION AS SHOWN IN THE FOLLOWING TABLE.  A=ABS(X)
C     AND C=SQRT(2).
C
C       FUNCTION     A.LE.(0.47)                   A.GT.(0.47)
C       REQUIRED       (ALL X)          (NEGATIVE X)       (POSITIVE X)
C
C       ERF(X)         ERF(X)             ERFC(A)-1          1-ERFC(X)
C
C       ERFC(X)       1-ERF(X)            2-ERFC(A)           ERFC(X)
C
C       FREQ(X)   0.5+0.5*ERF(X/C)      0.5*ERFC(A/C)    1-0.5*ERFC(X/C)
C
C     ******************************************************************
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION P1(4),Q1(4), P2(8),Q2(8), P3(5),Q3(5)
      DATA P1/2.42667 95523 0532 D2,
     1        2.19792 61618 2942 D1,
     2        6.99638 34886 1914 D0,
     3       -3.56098 43701 8154 D-2/
      DATA Q1/2.15058 87586 9861 D2,
     1        9.11649 05404 5149 D1,
     2        1.50827 97630 4078 D1,
     3        1.00000 00000 0000 D0/
      DATA P2/3.00459 26102 0162 D2,
     1        4.51918 95371 1873 D2,
     2        3.39320 81673 4344 D2,
     3        1.52989 28504 6940 D2,
     4        4.31622 27222 0567 D1,
     5        7.21175 82508 8309 D0,
     6        5.64195 51747 8974 D-1,
     7       -1.36864 85738 2717 D-7/
      DATA Q2/3.00459 26095 6983 D2,
     1        7.90950 92532 7898 D2,
     2        9.31354 09485 0610 D2,
     3        6.38980 26446 5631 D2,
     4        2.77585 44474 3988 D2,
     5        7.70001 52935 2295 D1,
     6        1.27827 27319 6294 D1,
     7        1.00000 00000 0000 D0/
      DATA P3/-2.99610 70770 3542 D-3,
     1        -4.94730 91062 3251 D-2,
     2        -2.26956 59353 9687 D-1,
     3        -2.78661 30860 9648 D-1,
     4        -2.23192 45973 4185 D-2/
      DATA Q3/1.06209 23052 8468 D-2,
     1        1.91308 92610 7830 D-1,
     2        1.05167 51070 6793 D0,
     3        1.98733 20181 7135 D0,
     4        1.00000 00000 0000 D0/
      DATA CONST1/ 0.70710 67811 86548 D0/
C        ( CONST1= SQRT(1/2) )
      DATA CONST2/0.56418 95835 47756 D0/
C        ( CONST2= SQRT(1/PI) )
C
C     ******************************************************************
C
C  ENTRY POINTS. SET IENTRY (=1 FOR ERF, =2 FOR ERFC, =3 FOR FREQ).
C
      ENTRY ERF(X)
      IENTRY=1
      T=X
      A=ABS(T)
      IF(A.LE.6.D0) GO TO 11
      TMP=1.D0
      ERFXX=SIGN(TMP,X)
      RETURN
C
      ENTRY ERFC(X)
      IENTRY=2
      T=X
      IF(T.GE.-6.D0) GO TO 10
      ERFXX=2.D0
      RETURN
C
      ENTRY FREQ(X)
      IENTRY=3
      T=CONST1*X
      IF(T.LE.6.D0) GO TO 10
      ERFXX=1.D0
      RETURN
C
C  SELECT BASIC FUNCTION. SET IBASIC (=1 FOR ERF, =2 FOR ERFC).
C
   10 A=ABS(T)
   11 S=T**2
      IF(A.GT.0.47D0) GO TO 20
C
C  IBASIC=1.  SET Y=ERF(T).
C
      IBASIC=1
      Y=T*(P1(1)+S*(P1(2)+S*(P1(3)+S*P1(4) )))
     1   /(Q1(1)+S*(Q1(2)+S*(Q1(3)+S*Q1(4) )))
      GO TO 30
C
C  IBASIC=2. SET Y=ERFC(A).
C
   20 IBASIC=2
      IF(A.GT.4.D0) GO TO 21
      Y=EXP(-S)*(P2(1)+A*(P2(2)+A*(P2(3)+A*(P2(4)+A*(P2(5)+
     1        A*(P2(6)+A*(P2(7)+A*P2(8) )))))))
     2         /(Q2(1)+A*(Q2(2)+A*(Q2(3)+A*(Q2(4)+A*(Q2(5)+
     3        A*(Q2(6)+A*(Q2(7)+A*Q2(8) )))))))
      GO TO 30
C
   21 Y=0.0
      IF(A.GT.26.D0) GO TO 30
      R=1.D0/A
      U=R**2
      Y=R*EXP(-S)*( CONST2 +
     1     U*(P3(1)+U*(P3(2)+U*(P3(3)+U*(P3(4)+U*P3(5) ))))
     2      /(Q3(1)+U*(Q3(2)+U*(Q3(3)+U*(Q3(4)+U*Q3(5) )))) )
C
C  EXPRESS FINAL RESULT IN TERMS OF Y.
C
   30 IF(IENTRY.NE.1) GO TO 40
      IF(IBASIC.EQ.2) GO TO 31
      ERFXX=Y
      RETURN
   31 ERFXX=1.D0-Y
      IF(X.LT.0.D0) ERFXX=-ERFXX
      RETURN
C
   40 IF(IENTRY.NE.2) GO TO 50
      IF(IBASIC.EQ.2) GO TO 41
      ERFXX=1.D0-Y
      RETURN
   41 ERFXX=Y
      IF(X.LT.0.D0) ERFXX=2.D0-Y
      RETURN
C
   50 R=0.5D0*Y
      IF(IBASIC.EQ.2) GO TO 51
      ERFXX=0.5D0+R
      RETURN
   51 ERFXX=1.D0-R
      IF(X.LT.0.D0) ERFXX=R
      RETURN
      END



